// Generated by CoffeeScript 1.4.0
(function() {
  var BaseCacheProvider, EveApi, https, querystring, sax, _;

  querystring = require('querystring');

  https = require('https');

  BaseCacheProvider = require('./baseProvider');

  sax = require('sax');

  _ = require('underscore');

  /*
    EvEApi request class
    Always uses SSL to make requests. No support for EvEApi Proxy or other shit.
  */


  EveApi = (function() {
    /*
      Constructor
    
      parameters:
        server: api endpoint to contact
        cacheProvider: A provider to in memory cache or another provider.
                       Must extend BaseCacheProvider.
    */

    function EveApi(server, cacheProvider) {
      this.server = server != null ? server : 'api.eveonline.com';
      this.cacheProvider = cacheProvider != null ? cacheProvider : new BaseCacheProvider;
      if (this.server === '') {
        this.server = 'api.eveonline.com';
      }
      if (!(this.cacheProvider instanceof BaseCacheProvider)) {
        throw new Error('Providers must extend BaseCacheProvider!');
      }
    }

    /*
      Make a api request.
      Uses the configured @server and follows the configured ssl setting.
    
      The callback HAS to be passed, otherwise you will not get any data.
    
      If the call fails the callback will get called with an error object and null for data.
    
      parameters:
        options: options needed for the api call. (api key and other parameters)
                 Must provide at least 'scope' and 'api'.
        callback: callback to execute after the call completes
    */


    EveApi.prototype.call = function(options, callback) {
      var cachedResult, postData, req, response, url,
        _this = this;
      if (!(callback != null)) {
        throw new Error('Must provide a callback!');
      }
      if (!options) {
        callback('Must provide an options object!', null);
        return;
      }
      if (!(options.scope != null) || !(options.api != null)) {
        callback('Must provide scope and api in the options object!', null);
        return;
      }
      url = "/" + options.scope + "/" + options.api + ".xml.aspx";
      postData = querystring.stringify(options);
      cachedResult = this.cacheProvider.get(url + postData);
      if (cachedResult != null) {
        callback(null, cachedResult);
        return;
      }
      options = {
        hostname: this.server,
        port: 443,
        path: url,
        method: 'POST',
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded',
          'Content-Length': postData.length
        }
      };
      response = "";
      req = https.request(options, function(res) {
        console.log(res.statusCode);
        res.on('data', function(data) {
          return response += data;
        });
        return res.on('end', function() {
          return _this.parseXML(response, function(error, obj) {
            _this.cacheProvider.set(url + postData, obj, obj.cacheduntil);
            callback(null, obj);
          });
        });
      });
      req.on('error', function(error) {
        return callback(error, null);
      });
      req.write(postData);
      return req.end();
    };

    /*
      Takes in an XML string and tries to parse it into a JSON object.
    
      parameters:
        xmlString: the xml
    */


    EveApi.prototype.parseXML = function(xmlString, callback) {
      var currentNode, jsonObj, nodeRef, parser;
      if (!xmlString || xmlString === '') {
        callback('XML cannot be empty!', null);
        return;
      }
      parser = sax.parser(false, {
        trim: true,
        normalize: true,
        lowercasetags: true
      });
      jsonObj = {};
      currentNode = "";
      nodeRef = jsonObj;
      parser.onerror = function(err) {
        console.log(err);
        return callback(err, null);
      };
      parser.ontext = function(text) {
        return nodeRef[currentNode] = text;
      };
      parser.onopentag = function(node) {
        var attribName, attribValue, name, rowObj, _ref;
        name = node.name;
        if (node.attributes && node.attributes.name) {
          name = node.attributes.name;
        }
        if (node.name !== 'row') {
          currentNode = name;
          nodeRef[name] = {
            parent: nodeRef
          };
          nodeRef = nodeRef[name];
          if (node.name === 'rowset') {
            nodeRef['rows'] = [];
          }
        }
        if (node.attributes && _.size(node.attributes) > 0) {
          rowObj = {};
          _ref = node.attributes;
          for (attribName in _ref) {
            attribValue = _ref[attribName];
            if (node.name !== 'row') {
              nodeRef[attribName] = attribValue;
            } else {
              rowObj[attribName] = attribValue;
            }
          }
          if (node.name === 'row') {
            return nodeRef['rows'].push(rowObj);
          }
        }
      };
      parser.onclosetag = function(node) {
        if (node === 'row') {
          return;
        }
        if (_.size(nodeRef) === 2 && nodeRef[node]) {
          nodeRef.parent[node] = nodeRef[node];
        }
        return nodeRef = nodeRef.parent;
      };
      parser.onend = function() {
        return callback(null, nodeRef.eveapi);
      };
      parser.write(xmlString);
      return parser.close();
    };

    return EveApi;

  })();

  module.exports = EveApi;

}).call(this);
